/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Collections
*  SCOPE :              Collections common services
*  FILE :               colroot.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __colroot_HXX
#define __colroot_HXX

/* IMPORTS *******************************************************************/

/*****************************************************************************/

class Collection : public ObjectX {
friend class ColLeaf ;
friend class ColNode ;

protected :
  int         _card ;   // Cardinality of the collection

  // insert element directly in the leaf or node.
  // The result is a new leaf or node if explosion occurs, null else.
  Ref<Collection> insertDirect(const void*, int) ;

  // Remove element directly from the leaf or node.
  void removeDirect(int) ;

  // Manage explosion of a node or a leaf
  Ref<Collection> split(const void*, int) ;

  // Return leaf which contains 'ind'
  virtual Ref<ColLeaf> getLeaf(int ind) = 0 ;

  // Reset leaf-node after insertion of a new child
  virtual void newChild(const void*, int) = 0 ;

  // Reset leaf-node after deletion of a new child
  virtual void delChild(int) = 0 ;

  // Return a clone of the leaf-node
  virtual Ref<Collection> clone() = 0 ;
  
  // Size of an element of a collection
  virtual int sizeElem() = 0 ;

  // Maximum Number of element in a collection leaf or node
  virtual int maxElem() = 0 ;

  // Return current number of elements the leaf-node can store
  virtual int nbElem() = 0 ;

  // Return the address of th nth element of the leaf-node
  virtual ColElem* tab(int pos) = 0 ;

  // Insert an element in a sub-tree (without construction of new root)
  virtual Ref<Collection> insertTree(const void* atom, int ind) = 0 ;

  // Remove an element from a sub-tree (without deletion of the root)
  virtual void removeTree(int ind) = 0 ;

public :
  Collection() {} ;
  Collection(int c) : _card(c) {} ;
  virtual ~Collection() {} ;
  
  // Return cardinality of the collection
  int count() const { return _card ; } 

  // Reset the collection
  virtual void clear() = 0 ;                    

  // Get the nth elem of the collection  
  virtual void* getElem(int ind) = 0 ;  

  // Return leaf for scan. Set begin and end for this leaf
  virtual Ref<ColLeaf> scanLeaf(int, int&, int&) = 0 ; 

  // Insert element in collection. Result is the new root.(could be the same)
  virtual Ref<Collection> insert(const void*, int) = 0 ; 

  // Remove the element. Result is the new root.(could be the same) 
  virtual Ref<Collection> remove(int) = 0 ; 

  // Usefull for dictionary. Return max key of the leaf or node
  virtual const yoo_ObjectX&  max() ; 

  // Usefull for dictionary. Compute indice for the given key
  virtual int indKey(const void* key,void* (*)(const void*),
		     int (*)(const void*,const void*));
} ;

#endif
